﻿using ComputerStoreContracts.BindingModels;
using ComputerStoreContracts.BusinessLogicContracts;
using ComputerStoreContracts.SearchModels;
using ComputerStoreContracts.StorageContracts;
using ComputerStoreContracts.ViewModels;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ComputerStoreBusinessLogic.BusinessLogic
{
    public class ComponentLogic : IComponentLogic
    {
        private readonly ILogger _logger;
        private readonly IComponentStorage _componentStorage;
        public ComponentLogic(ILogger<ComponentLogic> logger, IComponentStorage componentStorage)
        {
            _logger = logger;
            _componentStorage = componentStorage;
        }

        public bool Create(ComponentBindingModel model)
        {
            CheckModel(model);
            if(_componentStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }

        public bool Update(ComponentBindingModel model)
        {
            CheckModel(model);
            if (_componentStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }

        public bool Delete(ComponentBindingModel model)
        {
            CheckModel(model,false);
            _logger.LogInformation("Delete. ID:{ID}", model.ID);
            if (_componentStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }

        public ComponentViewModel? ReadElement(ComponentSearchModel model)
        {
            if(model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }

            _logger.LogInformation("ReadElement. ComponentName:{ComponentName}.ID:{ ID}", model.Name, model.ID);

            var element = _componentStorage.GetElement(model);

            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }

            _logger.LogInformation("ReadElement find. ID:{ID}", element.ID);
            return element;
        }

        public List<ComponentViewModel>? ReadList(ComponentSearchModel? model)
        {
            _logger.LogInformation("ReadList. ComponentName:{ComponentName}. ID:{ ID}", model?.Name, model?.ID);

            var list = model == null ? _componentStorage.GetFullList() : _componentStorage.GetFilteredList(model);

            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        private void CheckModel(ComponentBindingModel model, bool withParams = true)
        {
            if (model == null) { return; }
            if (!withParams) { return; }
            if(string.IsNullOrEmpty(model.Name)) { throw new ArgumentNullException("Invalid component's name", nameof(model)); }
            if(model.Price <= 0) { throw new ArgumentNullException("Invalid component's price", nameof(model)); }

            _logger.LogInformation("Component. ComponentName:{ComponentName}. Cost:{ Cost}. ID: { ID} ", model.Name, model.Price, model.ID);

            var element = _componentStorage.GetElement(new ComponentSearchModel
            {
                Name = model.Name
            });

            if (element != null && element.ID != model.ID)
            {
                throw new InvalidOperationException("Component with such name already exists.");
            }
        }
    }
}
