﻿using ComputerStoreBusinessLogic.OfficePackage;
using ComputerStoreBusinessLogic.OfficePackage.HelperModels;
using ComputerStoreContracts.BindingModels;
using ComputerStoreContracts.BusinessLogicContracts;
using ComputerStoreContracts.SearchModels;
using ComputerStoreContracts.StorageContracts;
using ComputerStoreContracts.ViewModels;
using ComputerStoreDataModels.Models;
using MigraDoc.DocumentObjectModel.Fields;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Threading.Tasks;

namespace ComputerStoreBusinessLogic.BusinessLogic
{
    public class EmployeeReportLogic : IEmployeeReportLogic
    {       
        private readonly IPCStorage _pcStorage;
        private readonly IProductStorage _productStorage;
        private readonly IConsignmentStorage _consignmentStorage;
        private readonly AbstractSaveToExcel _saveToExcel;
        private readonly AbstractSaveToPDF _saveToPDF;
        private readonly AbstractSaveToWord _saveToWord;

        public EmployeeReportLogic(IPCStorage pcStorage, IProductStorage productStorage, IConsignmentStorage consignmentStorage, AbstractSaveToPDF saveToPDF, AbstractSaveToWord saveToWord, AbstractSaveToExcel saveToExcel)
        {
            _pcStorage = pcStorage;
            _productStorage = productStorage;
            _consignmentStorage = consignmentStorage;
            _saveToPDF = saveToPDF;
            _saveToWord = saveToWord;
            _saveToExcel = saveToExcel;
        }

        public List<ReportPCByDateViewModel> GetPCsByDate(ReportDateBindingModel model)
        {
            return _pcStorage.GetFilteredList(new PCSearchModel
            {
                DateFrom = model.DateFrom,
                DateTo = model.DateTo
            })
            .Select(x => new ReportPCByDateViewModel
            {
                PCName = x.Name,
                Price = x.Price,
                Components = x.PCComponents.Values.Select(x => (x.Item1.Name, x.Item2)).ToList(),
                EmployeeUsername = x.EmployeeUsername
            }).ToList();

        }

        public List<ReportProductByDateViewModel> GetProductsByDate(ReportDateBindingModel model)
        {
            return _productStorage.GetFilteredList(new ProductSearchModel
            {
                DateFrom = model.DateFrom,
                DateTo = model.DateTo
            })
            .Select(x => new ReportProductByDateViewModel
            {
                ProductName = x.Name,
                EmployeeUsername = x.EmployeeUsername,
                Price = x.Price,
                Components = x.ProductComponents.Values.Select(x => (x.Item1.Name, x.Item2)).ToList()

            }).ToList();
        }

        public List<ReportConsignmentsViewModel> GetConsignmentsByComponents(ReportComponentsBindingModel model)
        {
            var list = new List<ReportConsignmentsViewModel>();

            Dictionary<int, IProductModel> specifiedProducts = new Dictionary<int, IProductModel>();
            
            foreach(var component in model.Components)
            {
                var products = _productStorage.GetFilteredList(new ProductSearchModel
                {
                    ComponentID = component.ID
                });

                foreach(var product in products)
                {
                    if(!specifiedProducts.ContainsKey(product.ID))
                    {
                        specifiedProducts.Add(product.ID, product);
                    }
                }
            }

            Dictionary<int, IConsignmentModel> specifiedConsignments = new Dictionary<int, IConsignmentModel>();

            foreach (var product in specifiedProducts)
            {
                var consignments = _consignmentStorage.GetFilteredList(new ConsignmentSearchModel
                {
                    ProductID = product.Key
                });

                if(!specifiedConsignments.Any())
                {
                    foreach (var consignment in consignments)
                    {
                        specifiedConsignments.Add(consignment.ID, consignment);
                    }
                }

                foreach(var consignment in consignments)
                {
                    if (!specifiedConsignments.ContainsKey(consignment.ID))
                    {
                        specifiedConsignments.Add(consignment.ID,consignment);
                    }
                }
                
            }

            foreach(var consignment in specifiedConsignments)
            {
                var record = new ReportConsignmentsViewModel
                {
                    ID = consignment.Key,
                    Products = consignment.Value.ConsignmentProducts.Values.Select(x => (x.Item1.Name,x.Item2)).ToList()
                };
                list.Add(record);
            }

            return list;
        }

       
        public void SavePCsAndProductsToPdfFile(ReportDateBindingModel model)
        {
            _saveToPDF.CreateDoc(new PDFInfo
            {
                FileName = model.FileName,
                Title = "Products 'n orders list",
                DateFrom = model.DateFrom!.Value,
                DateTo = model.DateTo!.Value,
                Products = GetProductsByDate(model),
                PCs = GetPCsByDate(model)
            });
        }

        public MemoryStream SaveConsignmentsToExcelFile(ReportComponentsBindingModel model)
        {
            return _saveToExcel.CreateReport(new ExcelInfo
            {
                FileName = model.FileName,
                Title = "Consignments list",
                ConsignmentProducts = GetConsignmentsByComponents(model)
            });
        }

        public MemoryStream SaveConsignmentsToWordFile(ReportComponentsBindingModel model)
        {
            return _saveToWord.CreateDoc(new WordInfo
            {
                FileName = model.FileName,
                Title = "Consignments list",
                Consignments = GetConsignmentsByComponents(model)
            });
        }
    }
}
