﻿using ComputerStoreContracts.BindingModels;
using ComputerStoreContracts.BusinessLogicContracts;
using ComputerStoreContracts.SearchModels;
using ComputerStoreContracts.StorageContracts;
using ComputerStoreContracts.ViewModels;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ComputerStoreBusinessLogic.BusinessLogic
{
    public class PCLogic : IPCLogic
    {
        private readonly ILogger _logger;
        private readonly IPCStorage _PCStorage;
        public PCLogic(ILogger<PCLogic> logger, IPCStorage PCStorage)
        {
            _logger = logger;
            _PCStorage = PCStorage;
        }

        public bool Create(PCBindingModel model)
        {
            CheckModel(model);
            if (_PCStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }

        public bool Update(PCBindingModel model)
        {
            CheckModel(model);
            if (_PCStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }

        public bool Delete(PCBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. ID:{ID}", model.ID);
            if (_PCStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }

        public PCViewModel? ReadElement(PCSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }

            _logger.LogInformation("ReadElement. PCName:{PCName}.ID:{ ID}", model.Name, model.ID);

            var element = _PCStorage.GetElement(model);

            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }

            _logger.LogInformation("ReadElement find. ID:{ID}", element.ID);
            return element;
        }

        public List<PCViewModel>? ReadList(PCSearchModel? model)
        {
            _logger.LogInformation("ReadList. PCName:{PCName}. ID:{ ID}", model?.Name, model?.ID);

            var list = model == null ? _PCStorage.GetFullList() : _PCStorage.GetFilteredList(model);

            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        private void CheckModel(PCBindingModel model, bool withParams = true)
        {
            if (model == null) { return; }
            if (!withParams) { return; }
            if (string.IsNullOrEmpty(model.Name)) { throw new ArgumentNullException("Invalid PC's name", nameof(model)); }
            if (model.Price <= 0) { throw new ArgumentNullException("Invalid PC's price", nameof(model)); }
            if (string.IsNullOrEmpty(model.EmployeeID.ToString())) { throw new ArgumentNullException("Invalid PC's employee ID", nameof(model)); }
            if (string.IsNullOrEmpty(model.RequestID.ToString())) { throw new ArgumentNullException("Invalid PC's request ID", nameof(model)); }

            _logger.LogInformation("PC. RequestID: {RequestID}. EmployeeID: {EmployeeID}. PCName:{PCName}. Cost:{ Cost}. ID: { ID} ",model.RequestID,model.EmployeeID, model.Name, model.Price, model.ID);

            var element = _PCStorage.GetElement(new PCSearchModel
            {
                Name = model.Name
            });

            if (element != null && element.ID != model.ID)
            {
                throw new InvalidOperationException("PC with such name already exists.");
            }
        }
    }
}
