﻿using ComputerStoreContracts.BindingModels;
using ComputerStoreContracts.BusinessLogicContracts;
using ComputerStoreContracts.SearchModels;
using ComputerStoreContracts.StorageContracts;
using ComputerStoreContracts.ViewModels;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ComputerStoreBusinessLogic.BusinessLogic
{
    public class ProductLogic : IProductLogic
    {
        private readonly ILogger _logger;
        private readonly IProductStorage _productStorage;
        public ProductLogic(ILogger<ProductLogic> logger, IProductStorage productStorage)
        {
            _logger = logger;
            _productStorage = productStorage;
        }

        public bool Create(ProductBindingModel model)
        {
            CheckModel(model);
            if (_productStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }

        public bool Update(ProductBindingModel model)
        {
            CheckModel(model);
            if (_productStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }

        public bool Delete(ProductBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. ID:{ID}", model.ID);
            if (_productStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }

        public ProductViewModel? ReadElement(ProductSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }

            _logger.LogInformation("ReadElement. ProductName:{ProductName}.ID:{ ID}", model.Name, model.ID);

            var element = _productStorage.GetElement(model);

            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }

            _logger.LogInformation("ReadElement find. ID:{ID}", element.ID);
            return element;
        }

        public List<ProductViewModel>? ReadList(ProductSearchModel? model)
        {
            _logger.LogInformation("ReadList. ProductName:{ProductName}. ID:{ ID}", model?.Name, model?.ID);

            var list = model == null ? _productStorage.GetFullList() : _productStorage.GetFilteredList(model);

            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        private void CheckModel(ProductBindingModel model, bool withParams = true)
        {
            if (model == null) { return; }
            if (!withParams) { return; }
            if (string.IsNullOrEmpty(model.Name)) { throw new ArgumentNullException("Invalid product's name", nameof(model)); }
            if (model.Price <= 0) { throw new ArgumentNullException("Invalid product's price", nameof(model)); }
            if (string.IsNullOrEmpty(model.EmployeeID.ToString())) { throw new ArgumentNullException("Invalid Product's employee ID", nameof(model)); }

            _logger.LogInformation("Product. EmployeeID: {EmployeeID}. product:{product}. Cost:{ Cost}. ID: { ID} ",model.EmployeeID, model.Name, model.Price, model.ID);

            var element = _productStorage.GetElement(new ProductSearchModel
            {
                Name = model.Name
            });

            if (element != null && element.ID != model.ID)
            {
                throw new InvalidOperationException("Product with such name already exists.");
            }
        }
    }
}
