﻿using ComputerStoreBusinessLogic.OfficePackage.HelperEnums;
using ComputerStoreBusinessLogic.OfficePackage.HelperModels;
using MigraDoc.DocumentObjectModel;
using MigraDoc.DocumentObjectModel.Tables;
using MigraDoc.Rendering;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ComputerStoreBusinessLogic.OfficePackage.Implements
{
    public class SaveToPDF : AbstractSaveToPDF
    {
        private Document? _document;
        private Section? _section;
        private Table? _table;

        private static ParagraphAlignment GetParagraphAlignment(PDFParagraphAlignmentType type)
        {
            return type switch
            {
                PDFParagraphAlignmentType.Center => ParagraphAlignment.Center,
                PDFParagraphAlignmentType.Left => ParagraphAlignment.Left,
                PDFParagraphAlignmentType.Right => ParagraphAlignment.Right,
                _ => ParagraphAlignment.Justify,
            };
        }
        private static void DefineStyles(Document document)
        {
            var style = document.Styles["Normal"];
            style.Font.Name = "Times New Roman";
            style.Font.Size = 14;
            style = document.Styles.AddStyle("NormalTitle", "Normal");
            style.Font.Bold = true;
        }

        protected override void CreateComplicatedRow(PDFRowParameters rowParameters, IEnumerable<(string, int)> components)
        {
            if (_table == null)
            {
                return;
            }
            var row = _table.AddRow();
            for (int i = 0; i < rowParameters.Texts.Count; ++i)
            {
                row.Cells[i].AddParagraph(rowParameters.Texts[i]);
                if (!string.IsNullOrEmpty(rowParameters.Style))
                {
                    row.Cells[i].Style = rowParameters.Style;
                }
                
                Unit borderWidth = 0.5;
                row.Cells[i].Borders.Left.Width = borderWidth;
                row.Cells[i].Borders.Right.Width = borderWidth;
                row.Cells[i].Borders.Top.Width = borderWidth;
                row.Cells[i].Borders.Bottom.Width = borderWidth;
                row.Cells[i].Format.Alignment = GetParagraphAlignment(rowParameters.ParagraphAlignment);
                row.Cells[i].VerticalAlignment = VerticalAlignment.Center;
                
            }
            foreach (var component in components)
            {
                row.Cells[rowParameters.Texts.Count].AddParagraph($"{component.Item1} {component.Item2}");
                if (!string.IsNullOrEmpty(rowParameters.Style))
                {
                    row.Cells[rowParameters.Texts.Count].Style = rowParameters.Style;
                }

                Unit borderWidth = 0.5;
                row.Cells[rowParameters.Texts.Count].Borders.Left.Width = borderWidth;
                row.Cells[rowParameters.Texts.Count].Borders.Right.Width = borderWidth;
                row.Cells[rowParameters.Texts.Count].Borders.Top.Width = borderWidth;
                row.Cells[rowParameters.Texts.Count].Borders.Bottom.Width = borderWidth;
                row.Cells[rowParameters.Texts.Count].Format.Alignment = GetParagraphAlignment(rowParameters.ParagraphAlignment);
                row.Cells[rowParameters.Texts.Count].VerticalAlignment = VerticalAlignment.Center;
            }
        }

        protected override void CreateParagraph(PDFParagraph PDFParagraph)
        {
            if (_section == null)
            {
                return;
            }
            var paragraph = _section.AddParagraph(PDFParagraph.Text);
            paragraph.Format.SpaceAfter = "1cm";
            paragraph.Format.Alignment = GetParagraphAlignment(PDFParagraph.ParagraphAlignment);
            paragraph.Style = paragraph.Style;
        }

        protected override void CreatePDF(PDFInfo info)
        {
            _document = new Document();
            DefineStyles(_document);
            _section = _document.AddSection();
        }

        protected override void CreateRow(PDFRowParameters rowParameters)
        {
            if (_table == null)
            {
                return;
            }
            var row = _table.AddRow();
            for (int i = 0; i < rowParameters.Texts.Count; ++i)
            {
                row.Cells[i].AddParagraph(rowParameters.Texts[i]);
                if (!string.IsNullOrEmpty(rowParameters.Style))
                {
                    row.Cells[i].Style = rowParameters.Style;
                }
                Unit borderWidth = 0.5;
                row.Cells[i].Borders.Left.Width = borderWidth;
                row.Cells[i].Borders.Right.Width = borderWidth;
                row.Cells[i].Borders.Top.Width = borderWidth;
                row.Cells[i].Borders.Bottom.Width = borderWidth;
                row.Cells[i].Format.Alignment = GetParagraphAlignment(rowParameters.ParagraphAlignment);
                row.Cells[i].VerticalAlignment = VerticalAlignment.Center;
            }
        }

        protected override void CreateTable(List<string> columns)
        {
            if (_document == null)
            {
                return;
            }
            _table = _document.LastSection.AddTable();
            foreach (var elem in columns)
            {
                _table.AddColumn(elem);
            }
        }

        protected override void SavePDF(PDFInfo info)
        {
            var renderer = new PdfDocumentRenderer(true)
            {
                Document = _document
            };

            renderer.RenderDocument();
            renderer.PdfDocument.Save(info.FileName);
        }
    }
}
